/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package hb.util;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import pendragon.Config;

public class HibernateUtil {
    private static SessionFactory sessionFactory;

    private static void __createSessionFactory(Configuration configuration) throws HibernateException {
	HibernateUtil.sessionFactory = configuration.addClass(hb.Tag.class)
						    .addClass(hb.File.class)
						    .addClass(hb.Catalogue.class)
						    .addClass(hb.DVD.class)
						    .addClass(hb.Image.class)
						    .setProperty("hibernate.current_session_context_class", "thread")
						    .setProperty("hibernate.show_sql", "false")
						    .setProperty("hibernate.dialect", "org.hibernate.dialect.HSQLDialect")
						    .setProperty("hibernate.connection.useUnicode", "true")
						    .setProperty("hibernate.connection.characterEncoding", "UTF-8")
						    .setProperty("hibernate.cache.provider_class", "org.hibernate.cache.NoCacheProvider")
						    .buildSessionFactory();
    }
    public static void createSessionFactory() {
	try {
	    HibernateUtil.__createSessionFactory(
		    new Configuration().setProperty("hibernate.connection.driver_class", Config.xmlConfig.getString("hibernate.driver_class", null))
		    		       .setProperty("hibernate.connection.url", Config.xmlConfig.getString("hibernate.url", null))
		    		       .setProperty("hibernate.connection.username", Config.xmlConfig.getString("hibernate.username", null))
		    		       .setProperty("hibernate.connection.password", Config.xmlConfig.getString("hibernate.password", null))
	    );
	} catch (Throwable ex) {
	    System.err.println("Initial SessionFactory creation failed." + ex);
	    throw new ExceptionInInitializerError(ex);
	}
    }
    public static SessionFactory getSessionFactory() {
	return sessionFactory;
    }
    public static void addAndCommit(Object o) {
	Session session = HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.save(o);
	session.getTransaction().commit();
	session.close();
    }
    public static void updateAndCommit(Object o) {
	Session session = HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.update(o);
	session.getTransaction().commit();
	session.close();
    }
    public static void removeAndCommit(Object o) {
	Session session = HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.delete(o);
	session.getTransaction().commit();
	session.close();
    }
    public static void refresh(Object o) {
	Session session = HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.refresh(o);
	session.getTransaction().commit();
	session.close();
    }
}
