/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon;

import static pendragon.i18n.I18n._;

import hb.util.HibernateUtil;

import java.io.File;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.hibernate.Session;

import pendragon.ui.config.ConfigWindow;

public class Config {
    public final static List<hb.Tag> embededTags = Arrays.asList(
	    new hb.Tag(_("Books"),       _("Some books."),        "books.png"),
	    new hb.Tag(_("CD disk"),     _("Data CD disk."),      "data-cd.png"),
	    new hb.Tag(_("DVD disk"),    _("Data DVD disk."),     "data-dvd.png"),
	    new hb.Tag(_("Documents"),   _("Some documents."),    "documents.png"),
	    new hb.Tag(_("Films"),       _("Some films."),        "films.png"),
	    new hb.Tag(_("Games"),       _("Some games."),        "games.png"),
	    new hb.Tag(_("Midi"),        _("Some midi files."),   "midis.png"),
	    new hb.Tag(_("Music"),       _("Some music."),        "musics.png"),
	    new hb.Tag(_("Photos"),      _("Some photos."),       "photos.png"),
	    new hb.Tag(_("Video music"), _("Some video music.") , "teledisks.png")
    	);
    
    public final static String PENDRAGON_DIR = ".pendragon" + File.separator;
    public final static String DATA_DIR = "data" + File.separator;
    public final static String TAG_ICONS_DIR = DATA_DIR + "tag-icons" + File.separator;

    public final static XMLConfiguration xmlConfig = new XMLConfiguration(new HierarchicalConfiguration());
    private static String xmlConfigFileName;
    private static String configDir;

    public static String getConfigDir() {
	return configDir;
    }
    public static void create() {
	String home = System.getProperty("user.home");
	File confDir = new File(home + File.separator + PENDRAGON_DIR);
	if (! confDir.exists()) confDir.mkdir();
	configDir = confDir.getPath() + File.separator;
	xmlConfigFileName = configDir + "config2.xml";
	try {
	    xmlConfig.load(xmlConfigFileName);
	} catch (Exception dontExists) {
	    try {
		xmlConfig.setEncoding("UTF-8");
		xmlConfig.setRootElementName("pendragon-config");
		xmlConfig.setProperty("config[@version]", "0.2");
		xmlConfig.save(xmlConfigFileName);
	    } catch (Exception cantCreate) {
		System.out.println(_("Unable to create config file."));
	    }
	}
	firstRun();
	startHSQLDBServer();
    }
    public static void close() {
	try {
	    xmlConfig.save(xmlConfigFileName);
	} catch (Exception e) {
	    System.out.println(_("Unable to create config file."));
	}
	stopHSQLDBServer();
    }
    public static void createMissingTags() {
	Session session = HibernateUtil.getSessionFactory().getCurrentSession();
	session.beginTransaction();		
	List<hb.Tag> fromTag = hb.util.MyQuery.createQuery("from Tag").list();
	for (hb.Tag tag: embededTags) {
	    boolean exists = false;
	    for (hb.Tag x: fromTag) {
		if (x.getName().equals(tag.getName())) {
		    exists = true;
		    break;
		}
	    }
	    if (! exists) session.save(tag);
	}
	session.getTransaction().commit();
    }
    public static boolean isEmbeded(String name) {
	for (hb.Tag tag: embededTags)
	    if (tag.getName().equals(name)) return true;
	return false;
    }
    public static Image getImage(Display display, String fileName) {
	Image image;
	if (icons.containsKey(fileName))
	    image = icons.get(fileName);
	else {
	    try {
		image = new Image(display, DATA_DIR + fileName);
		icons.put(fileName, image);
	    } catch (Exception e) {    
		image = null;
	    }
	}
	return image;
    }
    public static Image getTagImage(Display display, String fileName) {
	Image image = null;
	if (tagIcons.containsKey(fileName))
	    image = tagIcons.get(fileName);
	else {
	    try {
		image = new Image(display, TAG_ICONS_DIR + fileName);
		tagIcons.put(fileName, image);
	    } catch (Exception e) {
		hb.Image hbImage = hb.util.MyQuery
	    		.createQuery("from Image i where i.name=:name")
	    		.setParameter("name", fileName)
	    		.uniqueResult();
		if (hbImage != null) {
		    try {
			InputStream in = hbImage.getImage().getBinaryStream();
			image = new Image(display, in);
			in.close();
			tagIcons.put(fileName, image);
		    } catch (Exception se) {
			throw new RuntimeException(se);
		    }
		}
	    }
	}
	return image;
    }
    private static void firstRun() {
	if (! xmlConfig.getBoolean("firstrun[@passed]", false)) {
	    ConfigWindow win = new ConfigWindow(false);
	    win.open();
	    xmlConfig.setProperty("firstrun[@passed]", true);
	}
    }
    private static void startHSQLDBServer() {
	if (xmlConfig.getString("hibernate.driver_class", "").equals("org.hsqldb.jdbcDriver")) {
	    HSQLDBServer.runServer();
	    try {
		HSQLDBServer.createDatabase();
	    }
	    catch (Exception e) {
		throw new ExceptionInInitializerError(e); 
	    }
	}
    }
    private static void stopHSQLDBServer() {
	if (xmlConfig.getString("hibernate.driver_class", "").equals("org.hsqldb.jdbcDriver"))
	    HSQLDBServer.stopServer();
    }
    private final static HashMap<String, Image> icons = new HashMap<String, Image>();
    private final static HashMap<String, Image> tagIcons = new HashMap<String, Image>();
}
