/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import org.hsqldb.Server;

public class HSQLDBServer {
    public final static String DATABASE = "database";
    private static Server server;
    public static void runServer() {
	if (server == null) {
	    server = new Server();
	    server.setSilent(true);
	    server.setNoSystemExit(true);
	    server.setDatabasePath(0, getFullDBName());
	    server.setDatabaseName(0, "Pendragon");
	    Thread t = new Thread() {
		public void run() {
		    server.start();
		}
	    };
	    t.start();
	}
    }
    public static void stopServer() {
	//server.shutdown(); - czasami pojawiają się błędy
	try {
	    execute("shutdown");
	} catch (Exception e) {
	}
    }
    public static void createDatabase() throws Exception {
	File dataFile = new File(getFullDBName() + ".script");
	if (! dataFile.exists()) {
	    deleteFile(getFullDBName() + ".data");
	    deleteFile(getFullDBName() + ".properties");
	    deleteFile(getFullDBName() + ".log");
	    execute(hsqldb_sql);
	}
    }
    private static String getFullDBName() {
	return Config.getConfigDir() + DATABASE;
    }
    private static void deleteFile(String fileName) {
	File dataFile = new File(fileName);
	dataFile.delete();
    }
    private static void execute(String expr) throws Exception {
	Class.forName("org.hsqldb.jdbcDriver");
	Connection conn = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/Pendragon", "sa", "");
	Statement st = conn.createStatement();
	st.executeUpdate(expr);
	st.close();
	conn.close();
    }
    //
    private static String hsqldb_sql =
	"CREATE CACHED TABLE DVD ("+
	"  id           INT IDENTITY NOT NULL,"+
	"  name         VARCHAR(250) NOT NULL,"+
	"  description  VARCHAR,"+
	"  PRIMARY KEY (id)"+
	")"+

	"CREATE CACHED TABLE TAG ("+
	"  id           INT IDENTITY NOT NULL,"+
	"  name         VARCHAR(250) NOT NULL,"+
	"  description  VARCHAR,"+
	"  image        VARCHAR(64),"+
	"  PRIMARY KEY (id)"+
	")"+

	"CREATE CACHED TABLE TAG_DVD ("+
	"  id_tag       INT NOT NULL,"+
	"  id_dvd       INT NOT NULL,"+
	"  PRIMARY KEY (id_tag, id_dvd)"+
	")"+

	"CREATE CACHED TABLE CATALOGUE ("+
	"  id           INT IDENTITY NOT NULL,"+
	"  id_dvd       INT NOT NULL,"+
	"  id_catalogue INT,"+
	"  path         VARCHAR,"+
	"  PRIMARY KEY (id),"+
	"  FOREIGN KEY (id_dvd) REFERENCES DVD (id),"+
	"  FOREIGN KEY (id_catalogue) REFERENCES CATALOGUE (id)"+
	")"+

	"CREATE CACHED TABLE FILE ("+
	"  id           INT IDENTITY NOT NULL,"+
	"  id_catalogue INT NOT NULL,"+
	"  name         VARCHAR(250) NOT NULL,"+
	"  size         BIGINT,"+
	"  PRIMARY KEY (id),"+
	"  FOREIGN KEY (id_catalogue) REFERENCES CATALOGUE (id)"+
	")"+

	"CREATE CACHED TABLE IMAGE ("+
	"  id           INT IDENTITY NOT NULL,"+
	"  name         VARCHAR(250) NOT NULL,"+
	"  image        LONGVARBINARY NOT NULL,"+
	"  PRIMARY KEY (id)"+
	")";
}
