/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon;

import static pendragon.i18n.I18n._;

import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.hibernate.Session;

import pendragon.ui.wizards.ProgressWizardPage;

public class Tools {
    public static Image resizeImage(Image image, int width, int height) {
	Image scaled = new Image(Display.getDefault(), width, height);
	GC gc = new GC(scaled);
	gc.setAntialias(SWT.ON);
	gc.setInterpolation(SWT.HIGH);
	int org_width = image.getBounds().width;
	int org_height = image.getBounds().height;
	int delta = org_width - org_height;
	if (delta >= 0)
	    gc.drawImage(image, delta/2, 0, org_width - delta, org_height, 0, 0, width, height);
	else
	    gc.drawImage(image, 0, - delta/2, org_width, org_height - delta/2, 0, 0, width, height);
	image.dispose();
	gc.dispose();
	return scaled;
    }
    public static void saveImage(Image image, OutputStream out, boolean close) throws IOException {
	ImageLoader loader = new ImageLoader();
	loader.data = new ImageData[] { image.getImageData() };
	loader.save(out, SWT.IMAGE_PNG);
	if (close) out.close();
    }
    public static GridLayout createGridLayout(int numColumns, boolean makeColumnsEqualWidth) {
	GridLayout layout = new GridLayout(numColumns, makeColumnsEqualWidth);
	layout.horizontalSpacing = 0;
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	layout.verticalSpacing = 0;
	return layout;
    }
    public static hb.Image getHbImage(String name) {
	return hb.util.MyQuery.createQuery("from Image i where i.name=:name").setParameter("name", name).uniqueResult();
    }
    public static hb.DVD getHbDVD(String name) {
	return hb.util.MyQuery.createQuery("from DVD where name=:name").setParameter("name", name).uniqueResult();
    }
    public static hb.Tag getHbTag(String name) {
	return hb.util.MyQuery.createQuery("from Tag where name=:name").setParameter("name", name).uniqueResult();
    }
    public static void removeFromDatabase(hb.DVD dvd, ProgressWizardPage wizardPage) {
	wizardPage.setMessage(_("Removing. Can not stop this operation."));
	Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.update(dvd);
	for (hb.Catalogue catalogue: dvd.getCatalogues()) {
	    wizardPage.updateBar();
	    for (hb.File file: catalogue.getFiles()) {
		wizardPage.updateBar();
		session.delete(file);
	    }
	    session.delete(catalogue);
	}
	session.delete(dvd);
	session.getTransaction().commit();
	session.close();
    }
    public static void addToDatabase(hb.DVD dvd, ProgressWizardPage wizardPage) {
	wizardPage.setMessage(_("Adding."));
	Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	session.beginTransaction();
	session.save(dvd);
      petla:
	for (hb.Catalogue catalogue: dvd.getCatalogues()) {
	    if (wizardPage.isCaneled()) break petla;
	    wizardPage.updateBar();
	    session.save(catalogue);
	    for (hb.File file: catalogue.getFiles()) {
		if (wizardPage.isCaneled()) break petla;
		wizardPage.updateBar();
		session.save(file);
	    }
	}
	if (wizardPage.isCaneled()) session.getTransaction().rollback();
	else session.getTransaction().commit();
	session.close();
    }
}
