/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui;

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ExtendedModifyEvent;
import org.eclipse.swt.custom.ExtendedModifyListener;
import org.eclipse.swt.custom.PaintObjectEvent;
import org.eclipse.swt.custom.PaintObjectListener;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.GlyphMetrics;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Listener;

import pendragon.Config;
import pendragon.Tools;

public class CoolText extends Composite {
    private class ImageStore {
	public final Image image;
	public final int offset;
	ImageStore(Image image, int offset) {
	    this.image = image;
	    this.offset = offset;
	}
    }
    private final StyledText styledText;
    private Button textBold;
    private Button textItalic;
    private Button textStrikethrough;
    private Button textUnderline;
    private Button textFont;
    private FontData currentFontData;
    private final List<ImageStore> imageList = new LinkedList<ImageStore>();
    public CoolText(Composite parent, int style) {
	super(parent, SWT.NONE);
	this.setLayout(Tools.createGridLayout(1, false));
	if ((style & SWT.READ_ONLY) != SWT.READ_ONLY) {
	    Composite box = new Composite(this, SWT.NONE);
	    box.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	    box.setLayout(new RowLayout());
	    textBold = new Button(box, SWT.TOGGLE);
	    final Listener selected = new SelectedListener();
	    textBold.addListener(SWT.Selection, selected);
	    textBold.setImage(Config.getImage(getDisplay(), "format-text-bold.png"));
	    textItalic = new Button(box, SWT.TOGGLE);
	    textItalic.addListener(SWT.Selection, selected);
	    textItalic.setImage(Config.getImage(getDisplay(), "format-text-italic.png"));
	    textStrikethrough = new Button(box, SWT.TOGGLE);
	    textStrikethrough.addListener(SWT.Selection, selected);
	    textStrikethrough.setImage(Config.getImage(getDisplay(), "format-text-strikethrough.png"));
	    textUnderline = new Button(box, SWT.TOGGLE);
	    textUnderline.addListener(SWT.Selection, selected);
	    textUnderline.setImage(Config.getImage(getDisplay(), "format-text-underline.png"));
	    textFont = new Button(box, SWT.PUSH);
	    textFont.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    FontDialog dialog = new FontDialog(getShell(), SWT.NONE);
		    FontData fontData = dialog.open();
		    if (fontData != null) {
			String fontName = fontData.getName();
			if (fontName.equals("")) {
			    fontName = currentFontData.getName();
			    fontData.setName(fontName);
			}
			currentFontData = fontData;
			textFont.setText(fontName + " " + fontData.getHeight());
			selected.handleEvent(event);
		    }
		    textFont.pack();
		    styledText.setFocus();
		}
	    });
	}
	styledText = new StyledText(this, style);
	styledText.setLayoutData(new GridData(GridData.FILL_BOTH));
	styledText.addExtendedModifyListener(new ModifyListener());
	styledText.addListener(SWT.KeyDown, new CursorMovedListener());
	styledText.addListener(SWT.MouseDown, new MouseListener());
	styledText.addPaintObjectListener(new PaintImageListener());
	currentFontData = styledText.getFont().getFontData()[0];
	if (textFont != null) textFont.setText(currentFontData.getName() + " " + currentFontData.getHeight());
    }
    public String getText() {
	StringBuilder string = new StringBuilder();
	String text = styledText.getText();
	StyleRange currentStyle = null;
	boolean endStyle = false;
	for (int i = 0; i < text.length(); i++) {
	    StyleRange style = styledText.getStyleRangeAtOffset(i);
	    if (! styleCompare(currentStyle, style)) {
		if (endStyle) string.append("</style>");
		string.append("<style");
		FontData fontData = style.font.getFontData()[0];
		stringAppend(string, "font", fontData.getName());
		stringAppend(string, "size", "" + fontData.getHeight());
		stringAppend(string, "font-style", "" + fontData.getStyle());
		stringAppend(string, "strikeout", "" + style.strikeout);
		stringAppend(string, "underline", "" + style.underline);
		string.append(">");
		endStyle = true;
	    }
	    string.append(text.charAt(i));
	    currentStyle = style;
	}
	if (endStyle) string.append("</style>");
	return string.toString();
    }
    private void stringAppend(StringBuilder builder, String name, String text) {
	builder.append(" ");
	builder.append(name);
	builder.append("=\"");
	builder.append(text);
	builder.append("\"");
    }
    private boolean styleCompare(StyleRange a, StyleRange b) {
	if (a == null && b == null) return true;
	if (a == null || b == null) return false;
	if (a.strikeout != b.strikeout) return false;
	if (a.underline != b.underline) return false;
	FontData aData = a.font.getFontData()[0];
	FontData bData = b.font.getFontData()[0];
	if (aData.getStyle() != bData.getStyle()) return false;
	if (!aData.getName().equals(bData.getName())) return false;
	if (aData.getHeight() != bData.getHeight()) return false;
	return true;
    }
    public void setImages(Image... images) {
	imageList.clear();
	String text = styledText.getText();
	int lastOffset = 0;
	int offset;
	for (Image image: images) { 
	    offset = text.indexOf("\uFFFC", lastOffset);
	    imageList.add(new ImageStore(image, offset));
	    lastOffset = offset + 1;
	    StyleRange style = new StyleRange();
	    style.start = offset;
	    style.length = 1;
	    Rectangle rect = image.getBounds();
	    style.metrics = new GlyphMetrics(rect.height/2 + 5, rect.height/2 - 5, rect.width);
	    styledText.setStyleRange(style);
	}
    }
    public void setText(String string) {
	StringBuilder builder = new StringBuilder();
	LinkedList<StyleRange> styles = new LinkedList<StyleRange>();
	StringTokenizer tokens = new StringTokenizer(string, "<> =\"", true);
	StyleRange style = null;
	int start = 0;
	while (tokens.hasMoreTokens()) {
	    String s = tokens.nextToken();
	    if (s.equals("<")) {
		s = skipSpace(tokens);
		if (s.equals("style")) {
		    FontData fontData = null;
		    style = new StyleRange();
		    style.start = start;
		    style.length = 0;
		    while (true) {
			s = skipSpace(tokens);
			if (s.equals("font")) {
			    s = getTokenValue(tokens);
			    fontData = new FontData();
			    fontData.setName(s);
			}
			else if (s.equals("size")) {
			    s = getTokenValue(tokens);
			    fontData.setHeight(Integer.valueOf(s));
			}
			else if (s.equals("font-style")) {
			    s = getTokenValue(tokens);
			    fontData.setStyle(Integer.valueOf(s));
			}
			else if (s.equals("strikeout")) {
			    s = getTokenValue(tokens);
			    style.strikeout = Boolean.valueOf(s);
			}
			else if (s.equals("underline")) {
			    s = getTokenValue(tokens);
			    style.underline = Boolean.valueOf(s);
			}
			else if (s.equals(">")) {
			    if (fontData != null) style.font = new Font(getDisplay(), fontData);
			    styles.add(style);
			    break;
			}
			else throw new RuntimeException("Unknown token :" + s);
		    }
		}
		else if (s.equals("/style")) s = tokens.nextToken();
		else if (s.equals("image/")) {
		    s = tokens.nextToken();
		    builder.append("\uFFFC");
		    start += s.length();
		}
	    }
	    else {
		builder.append(s);
		start += s.length();
		if (style != null) style.length += s.length();
	    }
	}
	styledText.setText(builder.toString());
	styledText.setStyleRanges(styles.toArray(new StyleRange[0]));
	styledText.update();
    }
    private String skipSpace(StringTokenizer tokens) {
	String rval;
	while ((rval = tokens.nextToken()).equals(" "));
	return rval;
    }
    private String getTokenValue(StringTokenizer tokens) {
	String rval;
	skipSpace(tokens);
	skipSpace(tokens);
	rval = skipSpace(tokens);
	skipSpace(tokens);
	return rval;
    }
    private void updateControls(StyleRange style) {
    	textStrikethrough.setSelection(style.strikeout);
    	textUnderline.setSelection(style.underline);
    	FontData fontData = style.font.getFontData()[0];
    	textFont.setText(fontData.getName() + " " + fontData.getHeight());
    	textBold.setSelection((fontData.getStyle() & SWT.BOLD) == SWT.BOLD);
    	textItalic.setSelection((fontData.getStyle() & SWT.ITALIC) == SWT.ITALIC);	
    }
    private StyleRange createStyle(int start, int length) {
	StyleRange style = new StyleRange(start, length, null, null, SWT.NORMAL);
	int fontStyle = SWT.NORMAL;
	if (textBold.getSelection()) fontStyle |= SWT.BOLD;
	if (textItalic.getSelection()) fontStyle |= SWT.ITALIC;
	currentFontData.setStyle(fontStyle);
	style.font = new Font(getDisplay(), currentFontData);
	style.strikeout = textStrikethrough.getSelection();
	style.underline = textUnderline.getSelection();
	return style;
    }
    private class MouseListener implements Listener {
	@Override public void handleEvent(Event event) {
    	    try {
    		StyleRange style = styledText.getStyleRangeAtOffset(styledText.getCaretOffset());
    		updateControls(style);
    		currentFontData = style.font.getFontData()[0];
    	    }
    	    catch (Exception e) {
    	    }	    
	}
    }
    private class CursorMovedListener implements Listener {
	@Override public void handleEvent(Event event) {
	    switch (event.keyCode) {
	    	case SWT.ARROW_LEFT:
	    	case SWT.ARROW_RIGHT:
	    	case SWT.ARROW_UP:
	    	case SWT.ARROW_DOWN:
	    	    try {
	    		StyleRange style = styledText.getStyleRangeAtOffset(styledText.getCaretOffset());
	    		updateControls(style);
	    		currentFontData = style.font.getFontData()[0];
	    	    }
	    	    catch (Exception e) {
	    	    }
	    }
	}
    }
    private class ModifyListener implements ExtendedModifyListener {
	@Override public void modifyText(ExtendedModifyEvent event) {
	    if (event.length == 0) return;
	    if (event.length == 1) {
		StyleRange style = createStyle(event.start, event.length);
		styledText.setStyleRange(style);
	    }
	}
    }
    private class SelectedListener implements Listener {
	@Override public void handleEvent(Event event) {
	    styledText.setFocus();
	    Point p = styledText.getSelectionRange();
	    if (p.y > 0) {
		StyleRange style = createStyle(p.x, p.y);
		styledText.setStyleRange(style);
	    }
	}	
    }
    private class PaintImageListener implements PaintObjectListener {
	@Override public void paintObject(PaintObjectEvent event) {
	    GC gc = event.gc;
	    for (ImageStore store: imageList) {
		if (store.offset == event.style.start)
		    gc.drawImage(store.image, event.x, event.y + event.ascent - event.style.metrics.ascent);
	    }
	}
    }
}
