/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import pendragon.Config;

public class IconSet extends Composite {
    private static final String ICON_FILE_NAME = "IconFileName";
    private Button selected;
    public IconSet(Composite parent, int style) {
	super(parent, style);
	GridLayoutFactory.fillDefaults().numColumns(8).applyTo(this);
    }
    public IconSet(Composite parent, int style, boolean icons) {
	this(parent, style);
	if (icons) addDefaultSet();
    }
    public Button addIcon(String fileName) {
	Button radio = new Button(this, SWT.RADIO);
	radio.setImage(Config.getTagImage(this.getDisplay(), fileName));
	radio.setData(ICON_FILE_NAME, fileName);
	radio.addListener(SWT.Selection, new SelectListener());
	return radio;
    }
    public String getSelected() {
	if (selected == null) return null;
	String fileName = (String)selected.getData(ICON_FILE_NAME);
	if (fileName.equals("none.png")) return null;
	return fileName;
    }
    public boolean setSelected(String fileName) {
	if (fileName == null) return false;
	for (Control c: getChildren()) {
	    Button b = (Button)c;
	    if (fileName.equals(b.getData(ICON_FILE_NAME))) {
		selected.setSelection(false);
		b.setSelection(true);
		selected = b;
		return true;
	    }
	}
	return false;
    }
    @Override public void setEnabled(boolean enabled) {
	for (Control c: getChildren())
	    c.setEnabled(enabled);
    }
    @Override public boolean getEnabled() {
	Control[] c = getChildren();
	Button b = (Button)c[0];
	return b.getEnabled();
    }
    private void addDefaultSet() {
	selected = addIcon("none.png");
	selected.setSelection(true);
	addIcon("applications-education-mathematics.png");
	addIcon("ark.png");
	addIcon("audiocd.png");
//	addIcon("books.png");
	addIcon("books2.png");
	addIcon("books3.png");
	addIcon("chip.png");
	addIcon("contacts.png");
	addIcon("contacts2.png");
	addIcon("conversation.png");
//	addIcon("data-cd.png");
//	addIcon("data-dvd.png");
//	addIcon("documents.png");
	addIcon("favourite.png");
	addIcon("flag-green.png");
	addIcon("folder-blue.png");
//	addIcon("films.png");
//	addIcon("games.png");
	addIcon("icon-cartoon.png");
	addIcon("journal.png");
	addIcon("love.png");
	addIcon("mail.png");
	addIcon("mail2.png");
	addIcon("mail-message-new.png");
//	addIcon("midis.png");
	addIcon("misc.png");
//	addIcon("musics.png");
	addIcon("notes.png");
	addIcon("notes2.png");
	addIcon("package_wordprocessing.png");
	addIcon("paint.png");
	addIcon("password.png");
	addIcon("penguin.png");
//	addIcon("photos.png");
	addIcon("plot.png");
	addIcon("science.png");
	addIcon("source.png");
	addIcon("sourcecode.png");
	addIcon("stock_addressbook.png");
	addIcon("tar.png");
//	addIcon("teledisks.png");
	addIcon("tools.png");
	addIcon("toys.png");
	addIcon("tux guitariste.png");
    }
    private class SelectListener implements Listener {
	@Override public void handleEvent(Event event) {
	    selected = (Button)event.widget;
	}
    }
}
