/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.app;

import java.io.CharArrayWriter;
import java.io.PrintWriter;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IconAndMessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class ErrorDialog extends IconAndMessageDialog {
    private Throwable throwable;
    private String message;
    private Text details;
    private ErrorDialog(String message, Throwable throwable) {
	super((Shell)null);
	this.throwable = throwable;
	this.message = message;
    }
    public static int openError(String message, Throwable throwable) {
	ErrorDialog dialog = new ErrorDialog(message, throwable);
	return dialog.open();
    }
    @Override protected void createButtonsForButtonBar(Composite parent) {
	createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
	Button button = createButton(parent, IDialogConstants.DETAILS_ID, IDialogConstants.SHOW_DETAILS_LABEL, false);
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Button button = (Button)event.widget;
		toggleDetails();
		if (details != null) button.setText(IDialogConstants.HIDE_DETAILS_LABEL);
		else button.setText(IDialogConstants.SHOW_DETAILS_LABEL);
	    }
	});
    }
    @Override protected Control createDialogArea(Composite parent) {
	Composite composite = new Composite(parent, SWT.NONE);
	GridLayoutFactory.swtDefaults().numColumns(2).applyTo(composite);
	Label imgLabel = new Label(composite, SWT.NONE);
	imgLabel.setImage(getErrorImage());
	Label textLabel = new Label(composite, SWT.NONE);
	textLabel.setText(message);
	GridDataFactory.fillDefaults().grab(true, true).minSize(320, 60).applyTo(textLabel);
	return parent;
    }
    @Override protected Image getImage() {
	return null;
    }
    private void toggleDetails() {
	Point windowSize = getShell().getSize();
	Point oldSize = getShell().computeSize(SWT.DEFAULT, SWT.DEFAULT);
	if (details != null) {
	    details.dispose();
	    details = null;
	} else {
	    details = new Text((Composite)getContents(), SWT.MULTI | SWT.READ_ONLY | SWT.H_SCROLL | SWT.V_SCROLL);
	    GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 200).span(2, 1).applyTo(details);
	    getShell().layout();
	    printStackTrace();
	}
	Point newSize = getShell().computeSize(SWT.DEFAULT, SWT.DEFAULT);
	getShell().setSize(new Point(windowSize.x, windowSize.y + (newSize.y - oldSize.y)));
    }
    private void printStackTrace() {
	CharArrayWriter charWriter = new CharArrayWriter();
	PrintWriter out = new PrintWriter(charWriter);
	throwable.printStackTrace(out);
	details.setText(charWriter.toString());
    }
}
