/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.common;

import static pendragon.i18n.I18n._;
import hb.util.TagComparator;

import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.hibernate.Session;

import pendragon.Config;
import pendragon.ui.CoolText;

public class DescriptionBuilder {
	private hb.DVD dvd;
	private boolean showDVDName;
	public DescriptionBuilder(hb.DVD dvd) {
	    this(dvd, false);
	}
	public DescriptionBuilder(hb.DVD dvd, boolean showDVDName) {
	    this.dvd = dvd;
	    this.showDVDName = showDVDName;
	}
	private void appendText(StringBuilder builder, String text, String fontName, int size, int style) {
	    builder.append("<style");
	    builder.append(" font=\"" + fontName + "\"");
	    builder.append(" size=\"" + size + "\"");
	    builder.append(" font-style=\"" + style + "\">");
	    builder.append(text);
	    builder.append("</style>");
	}
	public void applyTo(CoolText coolText) {
	    StringBuilder builder = new StringBuilder();
	    FontData fontData = coolText.getFont().getFontData()[0];	    
	    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	    session.update(dvd);
	    if (showDVDName) {
		appendText(builder, _("Disc: "), fontData.getName(), fontData.getHeight() + 3, SWT.BOLD);
		appendText(builder, dvd.getName(), fontData.getName(), fontData.getHeight() + 3, SWT.ITALIC);
		builder.append("\n\n");
	    }
	    appendText(builder, _("Tags:   "), fontData.getName(), fontData.getHeight() + 3, SWT.BOLD);
	    LinkedList<Image> imageList = new LinkedList<Image>();
	    List<hb.Tag> tagList = Arrays.asList(dvd.getTags().toArray(new hb.Tag[0]));
	    Collections.sort(tagList, new TagComparator());
	    for (hb.Tag tag: tagList) {
		Image image = Config.getTagImage(Display.getDefault(), tag.getImage());
		if (image != null) imageList.add(image);
		else imageList.add(Config.getTagImage(Display.getDefault(), "none.png"));
		builder.append("<image/>");
		if (System.getProperty("os.name").equals("Linux")) appendText(builder, tag.getName() + "  ", fontData.getName(), fontData.getHeight(), SWT.ITALIC);
		else appendText(builder, tag.getName() + "  ", fontData.getName(), fontData.getHeight(), SWT.NORMAL);
	    }
	    builder.append("\n\n");
	    appendText(builder, _("Desc:   "), fontData.getName(), fontData.getHeight() + 3, SWT.BOLD);
	    builder.append(dvd.getDescription());
	    
	    coolText.setText(builder.toString());
	    coolText.setImages(imageList.toArray(new Image[0]));
	    session.close();
	}
}
