/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.common;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import pendragon.Config;
import pendragon.JFaceDragon;
import pendragon.ui.CoolText;

abstract public class View extends Composite {
    protected CTabFolder folder;
    protected ToolBar toolBar;
    protected ToolItem add;
    protected ToolItem remove;
    protected ToolItem edit;
    public View(Composite parent, int style) {
	super(parent, style);
	GridLayoutFactory.fillDefaults().applyTo(this);
	folder = new CTabFolder(this, SWT.BORDER);
	toolBar = new ToolBar(folder, SWT.FLAT | SWT.HORIZONTAL);
	createButtons(toolBar);
	folder.setTopRight(toolBar);
	folder.setSimple(false);
	folder.addListener(SWT.Selection, new Listener(){
	    @Override public void handleEvent(Event event) {
		Control c = folder.getSelection().getControl();
		if (c instanceof Selection) {
		    Selection s = (Selection)c;
		    s.select();
		}
	    }
	});
	GridDataFactory.fillDefaults().grab(true, true).applyTo(folder);
	buttonsSetEnabled(false);
    }
    public CTabItem createItem(String text, Control control) {
	return createItem(text, control, false);
    }
    public CTabItem createItem(String text, Control control, boolean showClose) {
	CTabItem item = new CTabItem(folder, SWT.NONE);
	item.setText(text);
	item.setControl(control);
	item.setShowClose(showClose);
	folder.setSelection(item);
	CoolText coolText = JFaceDragon.appWindow.getDescText();
	if (coolText != null) coolText.setText("");
	return item;
    }
    public void buttonsSetEnabled(boolean enabled) {
	remove.setEnabled(enabled);
	edit.setEnabled(enabled);
    }
    protected void createButtons(ToolBar toolBar) {
	add = new ToolItem(toolBar, SWT.PUSH);
	add.setImage(Config.getImage(getDisplay(), "small-list-add.png"));
	remove = new ToolItem(toolBar, SWT.PUSH);
	remove.setImage(Config.getImage(getDisplay(), "small-list-remove.png"));
	edit = new ToolItem(toolBar, SWT.PUSH);
	edit.setImage(Config.getImage(getDisplay(), "small-edit.png"));
	folder.setTabHeight(toolBar.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
    }
}
