/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.config;

import static pendragon.i18n.I18n._;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import pendragon.Config;

public class ConfigUI extends Composite {
    private final Composite main;
    private final Text host;
    private final Text port;
    private final Text user;
    private final Text password;
    private final Group group;
    private final Combo combo;
    private final Button check;
    public ConfigUI(Composite parent, int style) {
	super(parent, style);
	this.setLayout(new FillLayout());
	main = new Composite(this, SWT.NONE);
	GridLayout layout = new GridLayout();
	layout.numColumns = 2;
	main.setLayout(layout);
	new Label(main, SWT.NONE).setText(_("Database: "));
	combo = new Combo(main, SWT.READ_ONLY);
	combo.setItems(new String[] { "HSQLDB", "MySQL" });
	combo.select(0);
	combo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	combo.addSelectionListener(new SelectListener());
	check = new Button(main, SWT.CHECK);
	check.setText(_("Use defaults"));
	check.setSelection(true);
	check.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
	check.addSelectionListener(new SelectListener());
	group = new Group(main, SWT.NONE);
	group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
	group.setText(_("Settings"));
	group.setEnabled(false);
	group.setLayout(layout);
	new Label(group, SWT.NONE).setText("Host :");
	host = new Text(group, SWT.SINGLE | SWT.BORDER);
	host.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	new Label(group, SWT.NONE).setText("Port :");
	port = new Text(group, SWT.SINGLE | SWT.BORDER);
	port.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	new Label(group, SWT.NONE).setText(_("User name :"));
	user = new Text(group, SWT.SINGLE | SWT.BORDER);
	user.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	new Label(group, SWT.NONE).setText(_("Password :"));
	password = new Text(group, SWT.SINGLE | SWT.BORDER);
	password.setEchoChar('*');
	password.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	new Label(main, SWT.NONE);
	setDefaults();
	loadConfig();
    }
    public void saveConfig() {
	switch (combo.getSelectionIndex()) {
	    case 0:
		Config.xmlConfig.setProperty("hibernate.driver_class", "org.hsqldb.jdbcDriver");
	    	Config.xmlConfig.setProperty("hibernate.url", "jdbc:hsqldb:hsql://" + host.getText()+ ":" + port.getText() + "/Pendragon");
	    	break;
	    case 1:
		Config.xmlConfig.setProperty("hibernate.driver_class", "com.mysql.jdbc.Driver");
		Config.xmlConfig.setProperty("hibernate.url", "jdbc:mysql://" + host.getText()+ ":" + port.getText() + "/Pendragon");
		break;
	}
	Config.xmlConfig.setProperty("hibernate.username", user.getText());
	Config.xmlConfig.setProperty("hibernate.password", password.getText());
	Config.xmlConfig.setProperty("ui.config.host", host.getText());
	Config.xmlConfig.setProperty("ui.config.port", port.getText());
	Config.xmlConfig.setProperty("ui.config.database", combo.getSelectionIndex());
	Config.xmlConfig.setProperty("ui.config.check", check.getSelection());
    }
    public void loadConfig() {
	String s = Config.xmlConfig.getString("hibernate.username", null);
	if (s != null) user.setText(s);
	s = Config.xmlConfig.getString("hibernate.password", null);
	if (s != null) password.setText(s);
	s = Config.xmlConfig.getString("ui.config.host", null);
	if (s != null) host.setText(s);
	s = Config.xmlConfig.getString("ui.config.port", null);
	if (s != null) port.setText(s);
	int i = Config.xmlConfig.getInt("ui.config.database", -1);
	if (i != -1) combo.select(i);
	Boolean b = Config.xmlConfig.getBoolean("ui.config.check", true);
	check.setSelection(b);
	enableGroup(!b);
    }
    private void enableGroup(boolean enabled) {
	if (combo.getSelectionIndex() == 0) {
	    enabled = false;
	    check.setEnabled(false);
	} else check.setEnabled(true);
	group.setEnabled(enabled);
	for (Control c: group.getChildren()) c.setEnabled(enabled);
    }
    private void setDefaults() {
	switch(combo.getSelectionIndex()) {
	    case 0: setHSQLDBDefaults(); break;
	    case 1: setMySQLDefaults(); break;
	}
    }
    private void setMySQLDefaults() {
	host.setText("localhost");
	port.setText("3306");
	user.setText("root");
	password.setText("");
    }
    private void setHSQLDBDefaults() {
	host.setText("localhost");
	port.setText("9001");
	user.setText("sa");
	password.setText("");
    }
    private class SelectListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent e) {
	    enableGroup(!check.getSelection());
	    setDefaults();
	}
    }
}
