/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.config;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class ConfigWindow extends Dialog {
    private final boolean cancel_enabled;
    private ConfigUI ui;
    public ConfigWindow(boolean cancel_enabled) {
	super((Shell)null);
	setShellStyle(SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.MAX);
	this.cancel_enabled = cancel_enabled;
    }
    @Override protected void createButtonsForButtonBar(Composite parent) {
	super.createButtonsForButtonBar(parent);
	getButton(IDialogConstants.CANCEL_ID).setEnabled(cancel_enabled);
    }
    @Override protected Control createDialogArea(Composite parent) {
	ui = new ConfigUI(parent, SWT.None);
	GridDataFactory.fillDefaults().grab(true, true).applyTo(ui);
	return parent;
    }
    @Override protected Point getInitialSize() {
	Point size = super.getInitialSize();
	return new Point(size.x * 16/9, size.y);
    }
    @Override protected void okPressed() {
	ui.saveConfig();
	super.okPressed();
    }
    @Override public boolean close() {
	if (!cancel_enabled) {
	    if (getReturnCode() == IDialogConstants.OK_ID) return super.close();
	    else return false;
	} else return super.close();
    }
}
