/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.mainview;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import pendragon.Config;
import pendragon.ui.common.DescriptionBuilder;

public class MainModel {
    private String filePath;
    private hb.DVD dvd;
    private hb.Catalogue catalogue;
    private hb.File file;
    public MainModel(hb.DVD dvd) {
	this.dvd = dvd;
    }
    public MainModel(hb.Catalogue catalogue) {
	this.catalogue = catalogue;
    }
    public MainModel(hb.File file, boolean fullPath) {
	this.file = file;
	if (fullPath) {
	    StringBuilder builder = new StringBuilder();
	    getFullPath(builder, file.getCatalogue());
	    filePath = builder.toString();
	}
    }
    public MainModel(hb.File file) {
	this(file, false);
    }
    public Long getId() {
	if (dvd != null) return dvd.getId();
	if (catalogue != null) return catalogue.getId();
	if (file != null) return file.getId();
	return null;
    }
    public hb.Catalogue getParentCatalogue() {
	hb.Catalogue rootCatalogue = null;
	if (catalogue != null) rootCatalogue = catalogue.getCatalogue(); 
	if (file != null) rootCatalogue = file.getCatalogue();
	if (rootCatalogue != null) return rootCatalogue.getCatalogue();
	return null;
    }
    public hb.Catalogue getCatalogue() {
	return catalogue;
    }
    public hb.File getFile() {
	return file;
    }
    public Image getImage() {
	if (dvd != null) return Config.getImage(Display.getDefault(), "media-optical.png");
	if (catalogue != null) return Config.getImage(Display.getDefault(), "folder.png");
	if (file != null) return Config.getImage(Display.getDefault(), "unknown.png");
	return null;
    }
    public String getName() {
	if (dvd != null) return dvd.getName();
	if (catalogue != null) return catalogue.getPath();
	if (filePath != null) return filePath + file.getName();
	if (file != null) return file.getName();
	return null;
    }
    public String getFileSize() {
	if (file != null) return computeSize(file.getSize());
	return null;
    }
    public boolean isDVD() {
	return dvd != null;
    }
    public hb.DVD getDVD() {
	return dvd;
    }
    public DescriptionBuilder getDescriptionBuilder() {
	if (dvd == null) return null;
	return new DescriptionBuilder(dvd);
    }
    private String computeSize(long size) {
	if (size < 2048) return "" + size + " B";
	else if (size < 2097152) return "" + (long)size/1024 + " kB";
	else if (size < 2147483648L) return "" + (long)size/1048576 + " MB";
	return "" + (long)size/1073741824L + "GB";
    }
    private void getFullPath(StringBuilder builder, hb.Catalogue catalogue) {
	if (catalogue == null) return;
	hb.Catalogue c = catalogue.getCatalogue();
	getFullPath(builder, c);
	builder.append(catalogue.getPath());
	if (c != null) builder.append("/");
    }
}
