/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.mainview;

import hb.Catalogue;
import hb.util.CatalogueComparator;
import hb.util.DVDComparator;
import hb.util.FileComparator;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeSet;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.hibernate.Session;

import pendragon.Config;
import pendragon.JFaceDragon;
import pendragon.ui.CoolText;
import pendragon.ui.common.Selection;

public class MainTable extends Composite implements Selection {
    private TableData currentData;
    private final ToolItem home;
    private TableData mainData;
    private final ToolItem next;
    private final LinkedList<TableData> nextItems = new LinkedList<TableData>();
    private final ToolItem prev;
    private final LinkedList<TableData> prevItems = new LinkedList<TableData>();
    private MainModel selected = null;
    private final TableViewer table;
    private final HashMap<Long, TableData> tableMap = new HashMap<Long, TableData>();
    private final ToolItem up;

    public MainTable(Composite parent, int style) {
	super(parent, style);
	GridLayoutFactory.fillDefaults().spacing(0, 0).applyTo(this);
	ToolBar toolBar = new ToolBar(this, SWT.FLAT);
	home = new ToolItem(toolBar, SWT.PUSH);
	home.setImage(Config.getImage(getDisplay(), "go-home.png"));
	home.setEnabled(false);
	home.addListener(SWT.Selection, new Listener() {
	    @Override
	    public void handleEvent(Event event) {
		home.setEnabled(false);
		up.setEnabled(false);
		prevItems.add(currentData);
		setInputModel(mainData);
	    }
	});
	up = new ToolItem(toolBar, SWT.PUSH);
	up.setImage(Config.getImage(getDisplay(), "go-up.png"));
	up.setEnabled(false);
	up.addListener(SWT.Selection, new Listener() {
	    @Override
	    public void handleEvent(Event event) {
		hb.Catalogue catalogue = currentData.model[0]
			.getParentCatalogue();
		prevItems.add(currentData);
		nextItems.clear();
		next.setEnabled(false);
		if (catalogue != null)
		    setInputModel(createMainModel(catalogue));
		else {
		    up.setEnabled(false);
		    home.setEnabled(false);
		    setInputModel(mainData);
		}
	    }
	});
	prev = new ToolItem(toolBar, SWT.PUSH);
	prev.setImage(Config.getImage(getDisplay(), "go-previous.png"));
	prev.setEnabled(false);
	prev.addListener(SWT.Selection, new Listener() {
	    @Override
	    public void handleEvent(Event event) {
		TableData newModel = prevItems.removeLast();
		nextItems.add(currentData);
		next.setEnabled(true);
		if (prevItems.isEmpty()) {
		    setInputModel(mainData);
		    prev.setEnabled(false);
		} else
		    setInputModel(newModel);
		up.setEnabled(currentData != mainData);
		home.setEnabled(currentData != mainData);
	    }
	});
	next = new ToolItem(toolBar, SWT.PUSH);
	next.setImage(Config.getImage(getDisplay(), "go-next.png"));
	next.setEnabled(false);
	next.addListener(SWT.Selection, new Listener() {
	    @Override
	    public void handleEvent(Event event) {
		TableData newModel = nextItems.removeLast();
		if (nextItems.isEmpty())
		    next.setEnabled(false);
		prevItems.add(currentData);
		prev.setEnabled(true);
		setInputModel(newModel);
		up.setEnabled(currentData != mainData);
		home.setEnabled(currentData != mainData);
	    }
	});
	Composite composite = new Composite(this, SWT.NONE);
	GridDataFactory.fillDefaults().grab(true, true).applyTo(composite);
	TableColumnLayout tableLayout = new TableColumnLayout();
	composite.setLayout(tableLayout);
	table = new TableViewer(composite, SWT.BORDER | SWT.VIRTUAL
		| SWT.FULL_SELECTION);
	table.setContentProvider(new MainTableContentProvider(table));
	table.setLabelProvider(new MainTableLabelProvider());
	GridDataFactory.fillDefaults().grab(true, true).applyTo(
		table.getTable());

	final TableColumn column1 = new TableColumn(table.getTable(), SWT.LEFT);
	tableLayout.setColumnData(column1, new ColumnWeightData(85, true));
	final TableColumn column2 = new TableColumn(table.getTable(), SWT.RIGHT);
	tableLayout.setColumnData(column2, new ColumnWeightData(15, true));
	table.getTable().addListener(SWT.Selection, new SelectedListener());
	table.getTable().addMouseListener(new DoubleClickListener());
	createMainData(null, false);
    }
    public void createMainData(List<hb.DVD> fromDVD, boolean updateCoolText) {
	if (fromDVD == null)
	    mainData = new TableData(createMainModel(), -1);
	else
	    mainData = new TableData(createMainModel(fromDVD), -1);
	setInputModel(mainData);
	prevItems.clear();
	nextItems.clear();
	home.setEnabled(false);
	prev.setEnabled(false);
	next.setEnabled(false);
	up.setEnabled(false);
	if (this.selected != null) {
	    MainModel selected = null;
	    int index = 0;
	    for (MainModel data: mainData.model) {
		if (data.getName().equals(this.selected.getName())) {
		    selected = data;
		    break;
		}
		index++;
	    }
	    this.selected = selected;
	    if (selected != null) {
		table.getTable().select(index);
		if (updateCoolText) {
		    CoolText coolText = JFaceDragon.appWindow.getDescText();
		    selected.getDescriptionBuilder().applyTo(coolText);
		}
	    } else JFaceDragon.appWindow.getDescText().setText("");
	}
    }
    public void selectDVD(hb.DVD dvd) {
	int index = 0;
	for (MainModel model: mainData.model) {
	    if (model.getName().equals(dvd.getName())) {
		selected = model;
		table.getTable().select(index);
		CoolText coolText = JFaceDragon.appWindow.getDescText();
		selected.getDescriptionBuilder().applyTo(coolText);
		break;
	    }
	    index++;
	}
    }
    private MainModel[] createMainModel() {
	List<hb.DVD> fromDVD = hb.util.MyQuery.createQuery("from DVD").list();
	return createMainModel(fromDVD);
    }
    private TableData createMainModel(final hb.Catalogue catalogue) {
	Long id = catalogue.getId();
	if (tableMap.containsKey(id))
	    return tableMap.get(id);
	Session session = hb.util.HibernateUtil.getSessionFactory()
		.openSession();
	session.update(catalogue);
	TreeSet<hb.Catalogue> catalogueSet = new TreeSet<hb.Catalogue>(
		new CatalogueComparator());
	catalogueSet.addAll(catalogue.getCatalogues());
	TreeSet<hb.File> fileSet = new TreeSet<hb.File>(new FileComparator());
	fileSet.addAll(catalogue.getFiles());
	session.close();
	MainModel[] model = new MainModel[catalogueSet.size() + fileSet.size()];
	int index = 0;
	for (hb.Catalogue cat : catalogueSet)
	    model[index++] = new MainModel(cat);
	for (hb.File file : fileSet)
	    model[index++] = new MainModel(file);
	if (index == 0) {
	    model = new MainModel[1];
	    model[0] = new MainModel((hb.Catalogue) null) {
		@Override
		public Catalogue getParentCatalogue() {
		    return catalogue.getCatalogue();
		}
	    };
	}
	TableData tableIndex = new TableData(model, 0);
	tableMap.put(id, tableIndex);
	return tableIndex;
    }
    private MainModel[] createMainModel(List<hb.DVD> fromDVD) {
	MainModel[] model = new MainModel[fromDVD.size()];
	Collections.sort(fromDVD, new DVDComparator());
	int index = 0;
	for (hb.DVD dvd : fromDVD)
	    model[index++] = new MainModel(dvd);
	return model;
    }
    public MainModel getSelected() {
	int index = table.getTable().getSelectionIndex();
	if (index < 0)
	    return null;
	return currentData.model[index];
    }
    public void refresh() {
	if (currentData.model[0].isDVD()) {
	    Arrays.sort(currentData.model, new MainModelComparator());
	    if (selected != null) {
		table.getTable().deselectAll();
		int index = 0;
		for (MainModel model: currentData.model) {
		    if (model == selected) break;
		    index++;
		}
		System.out.println(index);
		table.refresh();
		table.getTable().select(index);
		CoolText coolText = JFaceDragon.appWindow.getDescText();
		selected.getDescriptionBuilder().applyTo(coolText);
	    }
	} else table.refresh();
    }
    @Override public void select() {
	CoolText coolText = JFaceDragon.appWindow.getDescText();
	if (selected != null)
	    selected.getDescriptionBuilder().applyTo(coolText);
	else
	    coolText.setText("");
    }
    private void setInputModel(TableData model) {
	currentData = model;
	table.setItemCount(0);
	table.setInput(model.model);
	table.setItemCount(currentData.model.length);
	if (currentData.index < 0)
	    table.getTable().deselectAll();
	else
	    table.getTable().select(currentData.index);
    }
    private class DoubleClickListener extends MouseAdapter {
	@Override public void mouseDoubleClick(MouseEvent event) {
	    hb.Catalogue catalogue;
	    int index = table.getTable().getSelectionIndex();
	    if (index < 0)
		return;
	    MainModel one = currentData.model[index];
	    if (one.isDVD())
		catalogue = hb.util.MyQuery
			.createQuery(
				"from Catalogue where id_dvd=:id_dvd and id_catalogue is null")
			.setParameter("id_dvd", one.getId()).uniqueResult();
	    else
		catalogue = one.getCatalogue();
	    if (catalogue != null) {
		currentData.index = index;
		prevItems.add(currentData);
		nextItems.clear();
		setInputModel(createMainModel(catalogue));
		up.setEnabled(true);
		home.setEnabled(true);
		prev.setEnabled(true);
		next.setEnabled(false);
	    }
	}
    }
    private class SelectedListener implements Listener {
	@Override public void handleEvent(Event event) {
	    CoolText coolText = JFaceDragon.appWindow.getDescText();
	    if (table.getTable().getSelectionCount() == 0) {
		coolText.setText("");
		return;
	    }
	    int index = table.getTable().getSelectionIndex();
	    if (index < 0)
		return;
	    MainModel one = currentData.model[index];
	    if (one.isDVD() && one != selected)
		one.getDescriptionBuilder().applyTo(coolText);
	    selected = one;
	    JFaceDragon.appWindow.getMainView().buttonsSetEnabled(true);
	}
    }
    private class TableData {
	public int index;
	public final MainModel[] model;

	public TableData(MainModel[] model, int index) {
	    this.model = model;
	    this.index = index;
	}
    }
}
