/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.mainview;

import static pendragon.i18n.I18n._;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import pendragon.Config;
import pendragon.JFaceDragon;
import pendragon.ui.WaitingDialog;
import pendragon.ui.common.View;
import pendragon.ui.wizards.AddEditDiskWizard;
import pendragon.ui.wizards.MyWizardDialog;
import pendragon.ui.wizards.RemoveDiskWizard;

public class MainView extends View {
    protected ToolItem search;
    private Menu searchMenu;
    private MainTable mainTable;
    private LinkedList<MenuItem> recentSearch = new LinkedList<MenuItem>();
    public MainView(Composite parent, int style) {
	super(parent, style);
	search.addSelectionListener(new SelectionAdapter() {
	    @Override public void widgetSelected(SelectionEvent e) {
		searchMenu.setVisible(true);
	    }
	});
	search.setToolTipText(_("Search"));
	searchMenu = createMenu();
	add.addSelectionListener(new AddListener());
	add.setToolTipText(_("Add new disk."));
	remove.addSelectionListener(new RemoveListener());
	remove.setToolTipText(_("Remove a disk."));
	edit.addSelectionListener(new EditListener());
	edit.setToolTipText(_("Edit a disk."));
	createItem(_("Catalog"), mainTable = new MainTable(folder, SWT.NONE));
	loadConfig();
    }
    public void setInputData(List<hb.DVD> fromDVD) {
	if (folder.getSelectionIndex() > 0) mainTable.createMainData(fromDVD, false);
	else mainTable.createMainData(fromDVD, true);
    }
    @Override protected void createButtons(ToolBar toolBar) {
	search = new ToolItem(toolBar, SWT.PUSH);
	search.setImage(Config.getImage(getDisplay(), "small-search.png"));
	ToolItem item = new ToolItem(toolBar, SWT.PUSH);
	item.setEnabled(false);
	super.createButtons(toolBar);
    }
    protected Menu createMenu() {
	final Menu menu = new Menu(getShell(), SWT.POP_UP);
	MenuItem item = new MenuItem(menu, SWT.PUSH);
	item.setText(_("New search"));
	item.addSelectionListener(new SearchListener());
	new MenuItem(menu, SWT.SEPARATOR);
	return menu;
    }
    private MenuItem createMenuItem(String text) {
	MenuItem item = __createMenuItem(text);
	if (recentSearch.size() > 10) recentSearch.removeFirst().dispose();
	String[] array = new String[recentSearch.size()];
	int index = 0;
	for (MenuItem i: recentSearch) array[index++] = i.getText();
	Config.xmlConfig.setProperty("Search.Item", array);
	return item;
    }
    private MenuItem __createMenuItem(String text) {
	MenuItem item = new MenuItem(searchMenu, SWT.NONE);
	item.setText(text);
	item.addSelectionListener(new SearchListener(text));
	recentSearch.add(item);
	return item;
    }
    private void loadConfig() {
	String[] array = Config.xmlConfig.getStringArray("Search.Item");
	for (String s: array) __createMenuItem(s);
    }
    private class SearchListener extends SelectionAdapter {
	private SearchTable searchTable;
	private String text;
	public SearchListener() {
	    this(null);
	}
	public SearchListener(String text) {
	    this.text = text;
	}
	@Override public void widgetSelected(SelectionEvent e) {
	    if (text == null) {
		InputDialog dialog = new InputDialog(null, _("Search"), _("Search"), "", null);
		if (dialog.open() == Dialog.OK && dialog.getValue().length() > 0) {
		    createSearchTable(dialog.getValue());
		    createMenuItem(dialog.getValue());
		    createItem(_("Results"), searchTable, true);
		}
	    } else {
		createSearchTable(text);
		createItem(_("Results"), searchTable, true);
	    }
	} 	
	private void createSearchTable(final String text) {
	    WaitingDialog w = new WaitingDialog(searchMenu.getShell());
	    w.open(new Runnable() {
		@Override public void run() {
		    final List<hb.File> files = hb.util.MyQuery
			.createQuery("from File where name like :name")
			.setParameter("name", "%" + text.replace(" ", "%") + "%")
			.list();
		    getDisplay().syncExec(new Runnable() {
			@Override public void run() {
			    searchTable = new SearchTable(folder, SWT.NONE, files);
			}
		    });
		    final MainModel[] model = searchTable.createMainModel();
		    getDisplay().syncExec(new Runnable() {
			@Override public void run() {
			    searchTable.setInputModel(model);
			}
		    });
		}
	    });
	}
    }
    private class AddListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent e) {
	    AddEditDiskWizard wizard = new AddEditDiskWizard(null);
	    WizardDialog dialog = new MyWizardDialog(null, wizard);
	    if (dialog.open() != Window.CANCEL) { // the disk is automaticly added to database
		JFaceDragon.appWindow.getTagView().refresh();
		JFaceDragon.appWindow.getTagView().updateMainTable();
		mainTable.selectDVD(wizard.getDVD()); 
	    }
	}
    }
    private class RemoveListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent event) {
	    MainModel model = mainTable.getSelected();
	    if (model == null) return;
	    if (! model.isDVD()) return;
	    WizardDialog dialog = new MyWizardDialog(null, new RemoveDiskWizard(model.getDVD()));
	    if (dialog.open() != Window.CANCEL) { // the disk is automaticly removed from database
		JFaceDragon.appWindow.getTagView().refresh();
		JFaceDragon.appWindow.getTagView().updateMainTable();
	    }
	}
    }
    private class EditListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent event) {
	    MainModel model = mainTable.getSelected();
	    if (model == null) return;
	    if (model.isDVD()) {
		AddEditDiskWizard wizard = new AddEditDiskWizard(model.getDVD());
		WizardDialog dialog = new MyWizardDialog(null, wizard);
		if (dialog.open() != Window.CANCEL) {
		    hb.util.HibernateUtil.updateAndCommit(wizard.getDVD());
		    JFaceDragon.appWindow.getTagView().refresh();
		    JFaceDragon.appWindow.getTagView().updateMainTable();
		}
	    }
	}
    }
}
