/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.mainview;

import hb.util.FileComparator;

import java.util.List;
import java.util.TreeSet;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableColumn;
import org.hibernate.Session;

import pendragon.JFaceDragon;
import pendragon.ui.CoolText;
import pendragon.ui.common.DescriptionBuilder;
import pendragon.ui.common.Selection;

public class SearchTable extends Composite implements Selection {
    private List<hb.File> files;
    private MainModel[] currentModel;
    private MainModel selected = null;
    private TableViewer table;
    public SearchTable(Composite parent, int style, List<hb.File> files) {
	super(parent, style);
	this.files = files;
	GridLayoutFactory.fillDefaults().spacing(0, 0).applyTo(this);
	Composite composite = new Composite(this, SWT.NONE);
	GridDataFactory.fillDefaults().grab(true, true).applyTo(composite);
	TableColumnLayout tableLayout = new TableColumnLayout();
	composite.setLayout(tableLayout);
	table = new TableViewer(composite, SWT.BORDER | SWT.VIRTUAL | SWT.FULL_SELECTION);
	table.setContentProvider(new MainTableContentProvider(table));
	table.setLabelProvider(new MainTableLabelProvider());
	GridDataFactory.fillDefaults().grab(true, true).applyTo(table.getTable());
	table.getTable().addListener(SWT.Selection, new SelectedListener());
	
	final TableColumn column1 = new TableColumn(table.getTable(), SWT.LEFT);
	tableLayout.setColumnData(column1, new ColumnWeightData(85, true));
	final TableColumn column2 = new TableColumn(table.getTable(), SWT.RIGHT);
	tableLayout.setColumnData(column2, new ColumnWeightData(15, true));
    }
    @Override public void select() {
	CoolText coolText = JFaceDragon.appWindow.getDescText();
	if (selected != null) new DescriptionBuilder(selected.getFile().getCatalogue().getDvd(), true).applyTo(coolText);
	else coolText.setText("");
    }
    public void setInputModel(MainModel[] model) {
	currentModel = model;
	table.setItemCount(0);
	table.setInput(model);
	table.setItemCount(model.length);
    }
    public MainModel[] createMainModel() {
	return createMainModel(files);
    }
    private MainModel[] createMainModel(List<hb.File> files) {
	TreeSet<hb.File> fileSet = new TreeSet<hb.File>(new FileComparator());
	fileSet.addAll(files);
	MainModel[] model = new MainModel[fileSet.size()];
	int index = 0;
	Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	for (hb.File file: fileSet) {
	    session.update(file);
	    model[index++] = new MainModel(file, true);
	}
	session.close();
	return model;
    }
    private class SelectedListener implements Listener {
	@Override public void handleEvent(Event event) {
	    CoolText coolText = JFaceDragon.appWindow.getDescText();
	    if (table.getTable().getSelectionCount() == 0) {
		coolText.setText("");
		return;
	    }
	    int index = table.getTable().getSelectionIndex();
	    if (index < 0) return;
	    MainModel one = currentModel[index];
	    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	    hb.File file = one.getFile();
	    session.update(file);
	    hb.DVD dvd = file.getCatalogue().getDvd();
	    session.close();
	    if (one != selected) new DescriptionBuilder(dvd, true).applyTo(coolText);
	    selected = one;
	}
    }
}
