/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.tagview;

import static pendragon.i18n.I18n._;
import java.util.Set;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import pendragon.Config;

public class TagModel {
    private hb.Tag tag;
    private int size;
    private boolean checked = false;
    private int index;
    public TagModel(hb.Tag tag, int index) {
	this.tag = tag;
	this.index = index;
	this.size = tag.getDvds().size();
    }
    public TagModel(int size, int index) {
	this.tag = null;
	this.size = size;
	this.index = index;
    }
    public void updateTag(hb.Tag tag) {
	this.tag = tag;
	this.size = tag.getDvds().size();	
    }
    public void updateSize(int size) {
	this.size = size;
    }
    public hb.Tag getTag() {
	return tag;
    }
    public Set<hb.DVD> getDVDs() {
	if (tag == null) return null;
	return tag.getDvds();
    }
    public boolean isEmbeded() {
	if (tag == null) return true;
	return Config.isEmbeded(tag.getName());
    }
    public String getName() {
	if (tag == null) return "*";
	else return tag.getName();
    }
    public int getCollectionSize() {
	return size;
    }
    public boolean getChecked() {
	return checked;
    }
    public void setChecked(boolean selected) {
	this.checked = selected;
    }
    public String getDescription() {
	if (tag == null) return _("\"One Ring to rule them all, One Ring to find them,\nOne Ring to bring them all and in the darkness bind them...\"");
	return tag.getDescription();
    }
    public Image getImage() {
	if (tag == null) return null;
	return Config.getTagImage(Display.getDefault(), tag.getImage());
    }
    public int getIndex() {
	return index;
    }
}
