/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.tagview;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.hibernate.Session;

import pendragon.Config;
import pendragon.JFaceDragon;

public class TagTable extends Composite {
    private TableViewer table;
    private TagModel[] model;
    public TagTable(Composite parent, int style) {
	super(parent, style);
	setLayout(new FillLayout());
	TagLabelProvider labelProvider = new TagLabelProvider();
	table = new TableViewer(this, SWT.BORDER | SWT.CHECK | SWT.FULL_SELECTION);
	table.setContentProvider(new TagContentProvider());
	table.getTable().addListener(SWT.Selection, new SelectionListener());
	ColumnViewerToolTipSupport.enableFor(table, ToolTip.NO_RECREATE);

	TableViewerColumn column = new TableViewerColumn(table, SWT.NONE);
	column.setLabelProvider(new DefaultCellLabelProvider(labelProvider) {
	    @Override public void update(ViewerCell cell) {
		TagModel model = (TagModel)cell.getElement();
		table.getTable().getItems()[model.getIndex()].setChecked(model.getChecked());
	    }
	});
	column = new TableViewerColumn(table, SWT.LEFT);
	column.setLabelProvider(new DefaultCellLabelProvider(labelProvider));
	column = new TableViewerColumn(table, SWT.RIGHT);
	column.setLabelProvider(new DefaultCellLabelProvider(labelProvider));

	createData();
    }
    public void createData() {
	TagModel[] oldModel = model;
	model = createModel();
	if (oldModel != null)
	    for (TagModel m: model)
		for (TagModel o: oldModel)
		    if (o.getName().equals(m.getName())) {
			m.setChecked(o.getChecked());
			break;
		    }
	table.setInput(model);
	table.getTable().setLinesVisible(true);
	for (TableColumn tableColumn: table.getTable().getColumns()) tableColumn.pack();	
    }
    public TagModel getSelected() {
	int index = table.getTable().getSelectionIndex();
	if (index < 0) return null;
	return model[index];
    }
    /*
    public void refresh() {
	List<hb.Tag> fromTag = hb.util.MyQuery.createQuery("from Tag").list();
	int index = 1;
	for (hb.Tag tag: fromTag) {
	    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	    session.update(tag);
	    tag.getDvds(); // lazy loading for future calls
	    int size = tag.getDvds().size(); // lazy loading for future calls
	    session.close();
	    if (! Config.isEmbeded(tag.getName()) || size > 0) model[index++].updateTag(tag);
	}
	Long l = hb.util.MyQuery.createQuery("select count(*) from DVD").uniqueResult();
	model[0].updateSize(l.intValue());
	table.refresh();
	for (TableColumn tableColumn: table.getTable().getColumns()) tableColumn.pack();
    }
    */
    private TagModel[] createModel() {
	List<hb.Tag> fromTag = hb.util.MyQuery.createQuery("from Tag").list();
	List<TagModel> listModel = new LinkedList<TagModel>();
	int index = 1;
	for (hb.Tag tag: fromTag) {
	    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	    session.update(tag);
	    int size = tag.getDvds().size(); // lazy loading for future calls
	    session.close();
	    if (! Config.isEmbeded(tag.getName()) || size > 0) listModel.add(new TagModel(tag, index++));
	}
	Long l = hb.util.MyQuery.createQuery("select count(*) from DVD").uniqueResult();
	TagModel all = new TagModel(l.intValue(), 0);
	all.setChecked(true);
	listModel.add(0, all);
	return listModel.toArray(new TagModel[0]);
    }
    public void updateMainTable() {
	if (model[0].getChecked()) JFaceDragon.appWindow.getMainView().setInputData(null);
	else {
	    HashMap<String, hb.DVD> dvdMap = new HashMap<String, hb.DVD>();
	    for (TagModel one: model)
		if (one.getChecked()) {
		    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
		    session.update(one.getTag());
		    for (hb.DVD dvd: one.getDVDs()) {
			String name = dvd.getName();
			if (! dvdMap.containsKey(name)) dvdMap.put(name, dvd);
		    }
		    session.close();
		}
	    JFaceDragon.appWindow.getMainView().setInputData(new ArrayList<hb.DVD>(dvdMap.values()));
	}
    }
    private class SelectionListener implements Listener {
	@Override public void handleEvent(Event event) {
	    TableItem[] items = table.getTable().getItems();
	    if (items[0].getChecked() != model[0].getChecked()) {
		for (TagModel one: model) one.setChecked(false);
		model[0].setChecked(true);
		updateMainTable();
	    } else {
		boolean changed = false;
		boolean zero = true;
		for (int i = 1; i < model.length; i++) {
		    if (model[i].getChecked() != items[i].getChecked()) {
			changed = true;
			model[i].setChecked(items[i].getChecked());
		    }
		    if (items[i].getChecked()) zero = false;
		}
		if (zero) {
		    model[0].setChecked(true);
		    updateMainTable();
		} else if (changed) {
		    model[0].setChecked(false);
		    updateMainTable();
		}
	    }
	    table.refresh();
	    int index = table.getTable().getSelectionIndex();
	    if (index >=0) JFaceDragon.appWindow.getTagView().buttonsSetEnabled(! model[index].isEmbeded());
	    else JFaceDragon.appWindow.getTagView().buttonsSetEnabled(false);
	}
    }
    private class DefaultCellLabelProvider extends CellLabelProvider {
	private ITableLabelProvider labelProvider;
	public DefaultCellLabelProvider(ITableLabelProvider labelProvider) {
	    super();
	    this.labelProvider = labelProvider;
	}
	@Override public String getToolTipText(Object element) {
	    TagModel model = (TagModel)element;
	    return model.getDescription();
	}
	@Override public Point getToolTipShift(Object object) {
		return new Point(5, 5);
	}
	@Override public int getToolTipDisplayDelayTime(Object object) {
		return 0;
	}
	@Override public int getToolTipTimeDisplayed(Object object) {
		return 50000;
	}
	@Override public Image getToolTipImage(Object object) {
	    TagModel model = (TagModel)object;
	    return model.getImage();
	}
	@Override public void update(ViewerCell cell) {
	    TagModel model = (TagModel)cell.getElement();
	    cell.setText(labelProvider.getColumnText(model, cell.getColumnIndex()));
	}
    }
}
