/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.tagview;

import static pendragon.i18n.I18n._;

import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;

import pendragon.Tools;
import pendragon.ui.common.View;
import pendragon.ui.wizards.AddEditTagWizard;
import pendragon.ui.wizards.MyWizardDialog;

public class TagView extends View {
    private Composite parent;
    private TagTable tagTable;
    public TagView(Composite parent, int style) {
	super(parent, style);
	this.parent = parent;
	add.addSelectionListener(new AddListener());
	add.setToolTipText(_("Add new TAG."));
	remove.addSelectionListener(new RemoveListener());
	remove.setToolTipText(_("Remove a TAG."));
	edit.addSelectionListener(new EditListener());
	edit.setToolTipText(_("Edit a TAG."));
	createItem(_("Tags"), tagTable = new TagTable(folder, SWT.NONE));
    }
    public void updateMainTable() {
	tagTable.updateMainTable();
    }
    public void refresh() {
	tagTable.createData();
    }
    private class AddListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent e) {
	    AddEditTagWizard wizard = new AddEditTagWizard(null);
	    WizardDialog dialog = new MyWizardDialog(null, wizard);
	    if (dialog.open() != Window.CANCEL) {
		hb.util.HibernateUtil.addAndCommit(wizard.getTag());
		tagTable.createData();
	    }
	}
    }
    private class RemoveListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent event) {
	    TagModel model = tagTable.getSelected();
	    if (model != null) {
		if (model.getDVDs().size() == 0) {
		    MessageBox box = new MessageBox(parent.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
		    box.setText("Pytanie");
		    box.setMessage(_("Do you want to delete this TAG ?"));
		    if (box.open() == SWT.YES) {
			    hb.Tag tag = model.getTag();
			    hb.Image image = Tools.getHbImage(tag.getImage());
			    if (image != null) hb.util.HibernateUtil.removeAndCommit(image);
			    hb.util.HibernateUtil.removeAndCommit(tag);
			    tagTable.createData();
			    remove.setEnabled(false);
			    edit.setEnabled(false);
		    }
		} else {
		    MessageBox box = new MessageBox(parent.getShell(), SWT.ICON_INFORMATION | SWT.OK);
		    box.setText(_("Information."));
		    box.setMessage(_("This TAG can not be deleted, because it is in use."));
		    box.open();
		}
	    }
	}
    }
    private class EditListener extends SelectionAdapter {
	@Override public void widgetSelected(SelectionEvent event) {
	    TagModel model = tagTable.getSelected();
	    if (model != null) {
		AddEditTagWizard wizard = new AddEditTagWizard(model.getTag());
		WizardDialog dialog = new MyWizardDialog(null, wizard);
		if (dialog.open() != Window.CANCEL) {
		    refresh();
		    hb.util.HibernateUtil.updateAndCommit(wizard.getTag());
		}
	    }
	}
    }
}
