/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.wizards;

import static pendragon.i18n.I18n._;
import java.io.File;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.hibernate.Session;

import pendragon.Config;
import pendragon.Tools;
import pendragon.ui.CoolText;
import pendragon.ui.TagIconSet;

public class AddEditDiskWizard extends MyWizard {
    private boolean canFinish = true;
    private boolean canCancel = true;
    private FirstPage firstPage;
    private SecondPage secondPage;
    private ProgressWizardPage progressPage;
    private EmptyWizardPage finalPage = new EmptyWizardPage(_("Add new disk."));
    private hb.DVD dvd;
    public AddEditDiskWizard(hb.DVD dvd) {
	if (dvd == null) {
	    firstPage = new FirstPage(_("Add new disk."));
	    secondPage = new SecondPage(_("Add new disk"));
	    progressPage = new ProgressWizardPage(_("Adding a disk to database."));
	    this.dvd = new hb.DVD();
	} else {
	    firstPage = new FirstPage(_("Edit an entry."));
	    secondPage = new SecondPage(_("Edit an entry."));
	    progressPage = null;
	    this.dvd = dvd;
	}
    }
    @Override public void addPages() {
	addPage(firstPage);
	addPage(secondPage);
	if (progressPage != null) addPage(progressPage);
	addPage(finalPage);
    }
    @Override public boolean canFinish() {
	return canFinish;
    }
    @Override public boolean canCancel() {
	return canCancel;
    }
    public hb.DVD getDVD() {
	return dvd;
    }
    @Override public boolean performCancel() {
	if (progressPage != null)
	    if (getContainer().getCurrentPage() == progressPage)
		if (! progressPage.isCaneled()) {
		    progressPage.setCancel();
		    return false;
		}
	return true;
    }
    @Override public boolean performFinish() {
	if (getContainer().getCurrentPage() == finalPage) return true;
	dvd.setName(firstPage.name.getText());
	dvd.setDescription(firstPage.desc.getText());
	Set<hb.Tag> tags = secondPage.tagSet.getSelected();
	dvd.setTags(new HashSet<hb.Tag>());
	if (tags != null) {
	    Session session = hb.util.HibernateUtil.getSessionFactory().openSession();
	    for (hb.Tag tag: tags) {
		session.update(tag);
		dvd.addToTag(tag);
	    }
	    session.close();
	}
	if (dvd.getId() == null) {
	    showProgressPage();
	    scan();
	    if (progressPage.isCaneled()) showFinalPage(_("The task has been canceled."));
	    else {
		Tools.addToDatabase(dvd, progressPage);
		if (progressPage.isCaneled()) showFinalPage(_("The task has been canceled."));
		else showFinalPage(_("The disk has been added to database."));
	    }
	    return false;
	}
	return true;
    }
    private void showProgressPage() {
	canFinish = false;
	getContainer().updateButtons();
	getContainer().showPage(progressPage);
	progressPage.update();	
    }
    private void showFinalPage(String message) {
	this.canFinish = true;
	this.canCancel = false;
	getContainer().updateButtons();
	getContainer().showPage(finalPage);
	finalPage.setMessage(message);
    }
    private void scan() {
	boolean isMount = false;
	boolean mount;
	if (! firstPage.mountButton.isDisposed()) mount = firstPage.mountButton.getSelection();
	else mount = false;
	File root = new File(firstPage.diskButton.getText());
	hb.Catalogue catalogue = new hb.Catalogue("/");
	catalogue.addToDVD(dvd);
	if (mount) {
	    progressPage.setMessage(_("Mounting a disk."));
	    isMount = mount();
	}
	progressPage.setMessage(_("Scanning."));
	createDisk(root.listFiles(), catalogue);
	if (isMount) {
	    progressPage.setMessage(_("Unmounting a disk."));
	    umount();
	}
	progressPage.setTextLabel("");
	progressPage.setMessage(_("Scanning finished."));
    }
    private void createDisk(File[] files, hb.Catalogue catalogue) {
	int i = 0;
	for (final File file: files) {
	    if (progressPage.isCaneled()) return;
	    if (i % 10 == 0) {
		progressPage.setTextLabel(file.getName());
		progressPage.updateBar();
	    }
	    i++;
	    if (file.isFile()) {
		hb.File newFile = new hb.File(file.getName(), file.length());
		newFile.addToCatalogue(catalogue);
	    }
	    else if (file.isDirectory()) {
		hb.Catalogue newCatalogue = new hb.Catalogue(file.getName());
		newCatalogue.addToDVD(this.dvd);
		newCatalogue.addToCatalogue(catalogue);
		createDisk(file.listFiles(), newCatalogue);
	    }
	}
    }
    private boolean mount() {
	Runtime runtime = Runtime.getRuntime();
	try {
	    Process process = runtime.exec("mount " + firstPage.diskButton.getText());
	    if (process.waitFor() == 0) return true;
	} catch (Exception e) {
	}
	return false;
    }
    private boolean umount() {
	Runtime runtime = Runtime.getRuntime();
	try {
	    Process process = runtime.exec("umount " + firstPage.diskButton.getText());
	    if (process.waitFor() == 0) return true;
	} catch (Exception e) {
	}
	return false;
    }
    private class FirstPage extends WizardPage {
	private final static String DISK_BUTTON = "DiskWizard.diskButton";
	private final static String MOUNT_BUTTON = "DiskWizard.mountButton";
	protected Button diskButton;
	protected Button mountButton;
	protected Text name;
	protected CoolText desc;
	protected FirstPage(String pageName) {
	    super(pageName);
	    setTitle(pageName);
	}
	@Override public void setErrorMessage(String newMessage) {
	    if (newMessage == null) canFinish = true;
	    else canFinish = false;
	    super.setErrorMessage(newMessage);
	    this.getWizard().getContainer().updateButtons();
	}
	@Override public void createControl(Composite parent) {
	    Composite composite = new Composite(parent, SWT.NONE);
	    composite.setLayout(new GridLayout(2, false));
	    new Label(composite, SWT.NONE).setText(_("Choose a disk:"));
	    diskButton = new Button(composite, SWT.PUSH);
	    if (System.getProperty("os.name").equals("Linux")) diskButton.setText(Config.xmlConfig.getString(DISK_BUTTON, "/mnt/cdrom"));
	    else diskButton.setText(Config.xmlConfig.getString(DISK_BUTTON, "d:\\"));
	    diskButton.addListener(SWT.Selection, new DiskListener());
	    diskButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    new Label(composite, SWT.NONE);
	    mountButton = new Button(composite, SWT.CHECK);
	    mountButton.setText(_("Automatically mount/unmount a disk."));
	    mountButton.setSelection(Config.xmlConfig.getBoolean(MOUNT_BUTTON, false));
	    mountButton.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    Config.xmlConfig.setProperty(MOUNT_BUTTON, mountButton.getSelection());
		}
	    });
	    if (dvd.getId() != null) mountButton.setEnabled(false);
	    if (! System.getProperty("os.name").equals("Linux")) {
		mountButton.dispose();
		new Label(composite, SWT.NONE);
	    }
	    new Label(composite, SWT.NONE).setText(_("Name:"));
	    name = new Text(composite, SWT.SINGLE | SWT.BORDER);
	    name.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    name.addListener(SWT.KeyUp, new KeyUpListener());
	    new Label(composite, SWT.NONE).setText(_("Description:"));
	    desc = new CoolText(composite, SWT.MULTI | SWT.WRAP | SWT.BORDER | SWT.V_SCROLL);
	    desc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	    setControl(composite);
	    setDVD(dvd);
	    if (dvd.getName() == null) {
		super.setErrorMessage(_("You have to give a name."));
		canFinish = false;
	    }
	}
	private void setDVD(hb.DVD dvd) {
	    if (dvd.getName() != null) name.setText(dvd.getName());
	    if (dvd.getDescription() != null) desc.setText(dvd.getDescription());
	}
	private class KeyUpListener implements Listener {
	    @Override public void handleEvent(Event event) {
		hb.DVD fromDVD = Tools.getHbDVD(name.getText());
		if (fromDVD != null) {
		    if (dvd.getName() == null) setErrorMessage(_("Given disk name already exist in database."));
		    else {
			if (dvd.getName().equals(name.getText())) setErrorMessage(null);
			else setErrorMessage(_("Given disk name already exist in database."));
		    }
		} else setErrorMessage(null);
		if (name.getText().length() == 0) setErrorMessage(_("You have to give a name."));
	    }
	}
	private class DiskListener implements Listener {
	    @Override public void handleEvent(Event event) {
		DirectoryDialog dialog = new DirectoryDialog(diskButton.getShell(), SWT.NONE);
		dialog.setFilterPath("/mnt");
		String fileName = dialog.open();
		if (fileName != null) {
		    diskButton.setText(fileName);
		    Config.xmlConfig.setProperty(DISK_BUTTON, fileName);
		}
	    }	    
	}
    }
    private class SecondPage extends WizardPage {
	protected TagIconSet tagSet;
	protected SecondPage(String pageName) {
	    super(pageName);
	    setTitle(pageName);
	}
	@Override public void createControl(Composite parent) {
	    tagSet = new TagIconSet(parent, SWT.NONE, 4);
	    tagSet.addDefults();
	    setControl(tagSet);
	    setDVD(dvd);
	}
	@Override public boolean canFlipToNextPage() {
	    return false;
	}
	private void setDVD(hb.DVD dvd) {
	    tagSet.setSelected(dvd.getTags());
	}
    }
}
