/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.wizards;

import static pendragon.i18n.I18n._;

import java.io.ByteArrayOutputStream;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.hibernate.lob.BlobImpl;

import pendragon.Config;
import pendragon.Tools;
import pendragon.ui.IconSet;

public class AddEditTagWizard extends MyWizard {
    private volatile boolean canFinish = true;
    private FirstPage firstPage;
    private SecondPage secondPage; 
    private hb.Tag tag;
    public AddEditTagWizard(hb.Tag tag) {
	if (tag == null) {
	    firstPage = new FirstPage(_("Add new TAG."));
	    secondPage = new SecondPage(_("Add new TAG."));
	    this.tag = new hb.Tag();
	}
	else {
	    firstPage = new FirstPage(_("Edit a TAG."));
	    secondPage = new SecondPage(_("Edit a TAG."));
	    this.tag = tag;
	}
    }
    @Override public void addPages() {
	addPage(firstPage);
	addPage(secondPage);
    }
    @Override public boolean canFinish() {
	return canFinish;
    }
    @Override public boolean performFinish() {
	tag.setName(firstPage.name.getText());
	tag.setDescription(firstPage.desc.getText());
	hb.Image hbImage = Tools.getHbImage(tag.getImage());
	if (hbImage != null) hb.util.HibernateUtil.removeAndCommit(hbImage);
	if (secondPage.icons.getEnabled()) tag.setImage(secondPage.icons.getSelected());
	else {
	    String name = Integer.toHexString(secondPage.iconFileImage.hashCode());
	    try {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		Tools.saveImage(secondPage.iconFileImage, out, true);
		hbImage = new hb.Image(name, new BlobImpl(out.toByteArray()));
		hb.util.HibernateUtil.addAndCommit(hbImage);
		tag.setImage(name);
	    } catch (Exception e) {
		throw new RuntimeException(e);
	    }
	}	
	return true;
    }
    public hb.Tag getTag() {
	return tag;
    }
    private class FirstPage extends WizardPage {
	protected Text name;
	protected Text desc;
	protected FirstPage(String title) {
	    super(title);
	    setTitle(title);
	}
	private void setTag(hb.Tag tag) {
	    if (tag.getName() != null) name.setText(tag.getName());
	    if (tag.getDescription() != null) desc.setText(tag.getDescription());
	}
	@Override public void setErrorMessage(String newMessage) {
	    if (newMessage == null) canFinish = true;
	    else canFinish = false;
	    super.setErrorMessage(newMessage);
	    this.getWizard().getContainer().updateButtons();
	}
	@Override public void createControl(Composite parent) {
	    Composite composite = new Composite(parent, SWT.NONE);
	    composite.setLayout(new GridLayout(2, false));
	    new Label(composite, SWT.NONE).setText(_("Name:"));
	    name = new Text(composite, SWT.SINGLE | SWT.BORDER);
	    name.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    name.addListener(SWT.KeyUp, new KeyUpListener());
	    new Label(composite, SWT.NONE).setText(_("Description:"));
	    desc = new Text(composite, SWT.MULTI | SWT.WRAP | SWT.BORDER | SWT.V_SCROLL);
	    desc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	    setControl(composite);
	    setTag(tag);
	    if (tag.getName() == null) {
		super.setErrorMessage(_("You have to give a name."));
		canFinish = false;
	    }
	}
	private class KeyUpListener implements Listener {
	    @Override public void handleEvent(Event event) {
		hb.Tag fromTag = Tools.getHbTag(name.getText());
		if (fromTag != null) {
		    if (tag.getName() == null) setErrorMessage(_("Given TAG already exist in database."));
		    else {
			if (tag.getName().equals(name.getText())) setErrorMessage(null);
			else setErrorMessage(_("Given TAG already exit in database."));
		    }
		} else setErrorMessage(null);
		if (name.getText().length() == 0) setErrorMessage(_("You have to give a name."));
	    }
	}
    }
    private class SecondPage extends WizardPage {
	protected IconSet icons;
	protected Button check;
	protected Button fileButton;
	protected Image iconFileImage;
	protected SecondPage(String title) {
	    super(title);
	    setTitle(title);
	}
	private void setTag(hb.Tag tag) {
	    String name = tag.getImage();
	    if (name == null) return;
	    if (! icons.setSelected(name)) {
		iconFileImage = Config.getTagImage(Display.getDefault(), name);
		fileButton.setImage(secondPage.iconFileImage);
		fileButton.setEnabled(true);
		icons.setEnabled(false);
		check.setSelection(true);
	    }
	}
	@Override public void createControl(Composite parent) {
	    Composite composite = new Composite(parent, SWT.NONE);
	    composite.setLayout(new GridLayout(2, false));
	    new Label(composite, SWT.NONE).setText(_("Icons:"));
	    icons = new IconSet(composite, SWT.NONE, true);
	    icons.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    check = new Button(composite, SWT.CHECK);
	    check.setText(_("From file:"));
	    check.addListener(SWT.Selection, new CheckListener());
	    fileButton = new Button(composite, SWT.PUSH);
	    fileButton.setText(_("Image from file ..."));
	    fileButton.setEnabled(false);
	    fileButton.setImage(new Image(parent.getDisplay(), Config.DATA_DIR + "none.png"));
	    fileButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	    fileButton.addListener(SWT.Selection, new OpenListener());      
	    setControl(composite);
	    setTag(tag);
	}
	private class CheckListener implements Listener {
	    @Override public void handleEvent(Event event) {
		boolean enabled = ((Button) event.widget).getSelection();
		icons.setEnabled(!enabled);
		fileButton.setEnabled(enabled);
	    }
	}
	private class OpenListener implements Listener {
	    @Override public void handleEvent(Event event) {
		FileDialog dialog = new FileDialog(fileButton.getShell(), SWT.OPEN);
		dialog.setFilterExtensions(new String[] { "*.png" });
		String fileName = dialog.open();
		if (fileName != null) {
		    iconFileImage = Tools.resizeImage(new Image(fileButton.getDisplay(), fileName), 48, 48);
		    fileButton.setImage(iconFileImage);
		}
	    }
	}
    }
}
