/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.wizards;

import static pendragon.i18n.I18n._;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

public class MyWizardDialog extends WizardDialog {
    private Button backButton;
    private Button nextButton;
    private Button cancelButton;
    private Button finishButton;
    private IMyWizard wizard;
    public MyWizardDialog(Shell parentShell, IMyWizard newWizard) {
	super(parentShell, newWizard);
	wizard = newWizard;
    }
    @Override protected void createButtonsForButtonBar(Composite parent) {
	((GridLayout) parent.getLayout()).makeColumnsEqualWidth = false;
	if (wizard.isHelpAvailable())
	    createButton(parent, IDialogConstants.HELP_ID, _("Help"), false);
	if (wizard.needsPreviousAndNextButtons())
	    createPreviousAndNextButtons(parent);
	finishButton = createButton(parent, IDialogConstants.FINISH_ID, _("Finish"), true);
	cancelButton = createCancelButton(parent);
	
	if (parent.getDisplay().getDismissalAlignment() == SWT.RIGHT) {
	    // Make the default button the right-most button.
	    // See also special code in org.eclipse.jface.dialogs.Dialog#initializeBounds()
	    finishButton.moveBelow(null);
	}
    }
    @Override public void updateButtons() {
	boolean canFlipToNextPage = false;
	boolean canFinish = wizard.canFinish();
	boolean canCancel = wizard.canCancel();
	if (backButton != null)
	    backButton.setEnabled(getCurrentPage().getPreviousPage() != null);
	if (nextButton != null) {
	    canFlipToNextPage = getCurrentPage().canFlipToNextPage();
	    nextButton.setEnabled(canFlipToNextPage);
	}
	finishButton.setEnabled(canFinish);
	cancelButton.setEnabled(canCancel);
	// finish is default unless it is disabled and next is enabled
	if (canFlipToNextPage && !canFinish)
	    getShell().setDefaultButton(nextButton);
	else
	    getShell().setDefaultButton(finishButton);
    }
    private Composite createPreviousAndNextButtons(Composite parent) {
	// increment the number of columns in the button bar
	((GridLayout) parent.getLayout()).numColumns++;
	Composite composite = new Composite(parent, SWT.NONE);
	// create a layout with spacing and margins appropriate for the font
	// size.
	GridLayout layout = new GridLayout();
	layout.numColumns = 0; // will be incremented by createButton
	layout.marginWidth = 0;
	layout.marginHeight = 0;
	layout.horizontalSpacing = 0;
	layout.verticalSpacing = 0;
	composite.setLayout(layout);
	GridData data = new GridData(GridData.HORIZONTAL_ALIGN_CENTER | GridData.VERTICAL_ALIGN_CENTER);
	composite.setLayoutData(data);
	composite.setFont(parent.getFont());
	backButton = createButton(composite, IDialogConstants.BACK_ID, _("< Back"), false);
	nextButton = createButton(composite, IDialogConstants.NEXT_ID, _("Next >"), false);
	return composite;
    }
    private Button createCancelButton(Composite parent) {
	// increment the number of columns in the button bar
	((GridLayout) parent.getLayout()).numColumns++;
	Button button = new Button(parent, SWT.PUSH);
	button.setText(_("Cancel"));
	setButtonLayoutData(button);
	button.setFont(parent.getFont());
	button.setData(new Integer(IDialogConstants.CANCEL_ID));
	button.addSelectionListener(new SelectionAdapter() {
	    @Override public void widgetSelected(SelectionEvent e) {
		cancelPressed();
	    }
	});
	return button;
    }
}
