/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.wizards;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

public class ProgressWizardPage extends WizardPage {
    private boolean canceled;
    private int value = 0;
    private ProgressBar bar;
    private Label textLabel;
    public ProgressWizardPage(String pageName) {
	super(pageName);
	setTitle(pageName);
    }
    public void setTextLabel(String text) {
	textLabel.setText(text);
	textLabel.pack();
	update();
    }
    public void updateBar() {
	bar.setSelection(((value++)/10) % bar.getMaximum());
	update();
    }
    public void update() {
	Shell shell = getContainer().getShell();
	shell.getDisplay().update();
	if (!shell.isDisposed())
	    while (shell.getDisplay().readAndDispatch());	
    }
    public void setCancel() {
	canceled = true;
    }
    public boolean isCaneled() {
	return canceled;
    }
    @Override public void createControl(Composite parent) {
	Composite composite = new Composite(parent, SWT.NONE);
	composite.setLayout(new GridLayout());
	textLabel = new Label(composite, SWT.NONE);
	textLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	bar = new ProgressBar(composite, SWT.SMOOTH);
	bar.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	setControl(composite);
	update();
    }
    @Override public void setMessage(String newMessage, int newType) {
	super.setMessage(newMessage, newType);
	update();
    }
    @Override public IWizardPage getPreviousPage() {
	return null;
    }
    @Override public boolean canFlipToNextPage() {
	return false;
    }
}
